/*
 * Copyright 2010-2011 Ning, Inc.
 *
 * Ning licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package com.ning.billing.entitlement.api.timeline;

import java.util.List;
import java.util.UUID;

import org.joda.time.DateTime;

import com.ning.billing.catalog.api.PlanPhaseSpecifier;
import com.ning.billing.entitlement.api.SubscriptionTransitionType;

public interface SubscriptionTimeline {

    public UUID getId();

    public List<DeletedEvent> getDeletedEvents();

    public List<NewEvent> getNewEvents();

    public List<ExistingEvent> getExistingEvents();

    public long getActiveVersion();

    public interface DeletedEvent {
        public UUID getEventId();
    }

    public interface NewEvent {
        public PlanPhaseSpecifier getPlanPhaseSpecifier();

        public DateTime getRequestedDate();

        public SubscriptionTransitionType getSubscriptionTransitionType();

    }

    public interface ExistingEvent extends DeletedEvent, NewEvent {
        public DateTime getEffectiveDate();

        public String getPlanPhaseName();
    }
}
